'***************************************************************************************************************************
'                                                  FTPD Search System
'***************************************************************************************************************************
'Copyright (C) 2006 Christer Finsen, cfinsenatcuifdotcx and Nils-Erik Aurker, nils-erikdotauraakerattoetipdotcom
'
'This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
'as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 
'
'This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
'of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
'
'You should have received a copy of the GNU General Public License along with this program; if not, write to the
'Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
'---------------------------------------------------------------------------------------------------------------------------

'SEE README.TXT FOR INSTALLATION

'******
'Prefix
'******
'a		= Array
'b		= Booalean
'c		= Collection
'f_		= Function
'i		= Integer
'o		= Object
's		= String
's_		= Sub
'CAPITAL	= Constant

Option Explicit

'Import Configuration
Import ("Search.Config")

'Make sure constants in include_search are defined
If CONVERT = "" Or REPLACEVFS = "" Or EVENTLOG = "" Or FILELOG = "" Or LOGS = "" _
Or LOCATE = "" Or UPDATEDB = "" Or OUTPUT = "" Or VFS = "" _
Or RESULTDEF = "" Or RESULTMAX = "" Or SEARCHTYPE = "" _
Or TypeName(SCENEVIEW) <> "Boolean" Or TypeName(WIDTH) <> "Integer" _
Or TypeName(SCENEHEADER) <> "String" Or TypeName(SCENESEPARATOR) <> "String" Or TypeName(SCENETAIL) <> "String" _
Or TypeName(NORMALHEADER) <> "String" Or TypeName(NORMALSEPARATOR) <> "String" Or TypeName(NORMALTAIL) <> "String" _
Or TypeName(EXCLUDEDVFS) <> "String" Or TypeName(SEARCHDETAILS) <> "Boolean" Then
  wscript.echo "One or more required constants is not defined. Please see readme.txt for installation"
  wscript.quit
End If

'***************************************************************************************************************************
'                                                CUSTOMIZABLE CONSTANTS
'***************************************************************************************************************************
'See Search.Config

'***************************************************************************************************************************
'                               DO NOT CHANGE THESE CONSTANTS UNLESS YOU KNOW WHAT YOU ARE DOING
'***************************************************************************************************************************
Const APPNAME       = "FTPD Search System"
Const APPVERMAJOR   = "2"
Const APPVERMINOR   = "3.4 (By TuS)"
Const FORREADING    = 1
Const FORWRITING    = 2
Const FORAPPENDING  = 8
Const SUCCESS       = 0
Const ERRORL        = 1
Const WARNING       = 2
Const INFORMATION   = 4
Const ABORT         = "Script aborted with error(s). To determine cause, see logged information"
Const SITECMD       = "search"

'Script Root folder
Dim SCRIPTPATH : SCRIPTPATH = Left(WScript.ScriptFullName, InStrRev(WScript.ScriptFullName,"\")-1)
Dim SCRIPTNAME : SCRIPTNAME = Left(WScript.ScriptName, InStrRev(WScript.ScriptName,".")-1)

Dim aDb(),aIndex(),aLog,aOut(),aOption
Dim bLog
Dim cFiles
Dim i,iCnt,iEnd,iLn,iRes,iOut,iStart
Dim oArgs,oFile,oFldr,oFso,oRegEx,oShell,oOut
Dim sCmd,sDBRoot,sErrLogRoot,sGCID,sLine,sLog,sLogRoot,sOutput,sSrchStr,sSrchType,sDBTime

'Create Shell object
Set oShell = Wscript.CreateObject("Wscript.Shell")

s_Log SUCCESS, "[Main Code] " & APPNAME & " " & APPVERMAJOR & "." &  APPVERMINOR & " started successfully"

'Create FileSystemObject
Set oFso = CreateObject("Scripting.FileSystemObject")

'Assign root of log
sLogRoot = SCRIPTPATH & "\SrchLog"

'Check log folder, abort if it doesn't exists and can't be created
s_Log INFORMATION, "[Main Code] Call f_CreateFolder, " & sLogRoot
If f_CreateFolder(sLogRoot) <> True Then
	s_TerminateInErr()
End If

'Set name of logfile
sLog = sLogRoot & "\" & f_GetDayTimeDate() & ".log"
If (oFso.FileExists(sLog)) Then
  s_Log INFORMATION, "[Main Code] The logfile : " & sLog & " allready exists. Appending random number to name of logfile"
  sLog = sLogRoot & "\" & f_GetDayTimeDate() & "_" & f_Randomize(999999) & ".log"
End If

s_Log INFORMATION, "[Main Code] Logfile = " & sLog

'Turning on filelog
bLog = True

'Assign root of error log path
sErrLogRoot = SCRIPTPATH & "\SrchErr"

'Check SrchErr, abort if it doesn't exists and can't be created
s_Log INFORMATION, "[Main Code] Call f_CreateFolder, " & sErrLogRoot
If f_CreateFolder(sErrLogRoot) <> True Then
	s_TerminateInErr()
End If

'Assign path to sOutput directory
sOutput = OUTPUT

'Check output directory, abort if it doesn't exists and can't be created
s_Log INFORMATION, "[Main Code] Call f_CreateFolder, " & sOutput
If f_CreateFolder(OUTPUT) <> True Then
	s_TerminateInErr()
End If

'Bind to arguments
Set oArgs = wscript.Arguments

s_Log INFORMATION, "[Main Code] Number of Arguments : " & oArgs.Count

For i = 0 To oArgs.Count -1
	s_Log INFORMATION, "[Main Code] Argument#" & i & ",Value=" & oArgs(i)
Next

'Simple verification that the FTPD process is initiating the command in a way we expect
If oArgs.Count < 3 Then
  s_Log ERRORL, "[Main Code] FTPD or this installation has been tamper with. This command needs at least three arguments to run"
  s_TerminateInSysErr()
End If

'Assign Global Unique Client ID from argument 0
sGCID = oArgs(0)
s_Log INFORMATION, "[Main Code] sGCID=" & sGCID

'Simple verification that the FTPD process is initiating the command in a way we expect
If oArgs(1) <> SITECMD Then
  s_Log ERRORL, "[Main Code] FTPD or this installation has been tamper with. Expected : " & SITECMD & " , Found : " & oArgs(1)
  s_TerminateInSysErr()
End If

'Create output file (FTPD Reply)
s_CreateOutObj()

'Check if user has requested help
If LCase(oArgs(2)) = "help" Or LCase(oArgs(2)) = "-h" Or LCase(oArgs(2)) = "-help" Then
  s_Log INFORMATION, "[Main Code] User has requested help, call s_TerminateInHlp()"
  s_TerminateInHlp()
End If

'Check if user has requested examples
If LCase(oArgs(2)) = "showme" Or LCase(oArgs(2)) = "-showme" Then
  s_Log INFORMATION, "[Main Code] User has requested examples, call s_TerminateInShowMe()"
  s_TerminateInShowMe()
End If

If oArgs.Count > 3 Then
  s_Log INFORMATION, "[Main Code] Options are provided, start sub analyze"

  'If several options provided, ReDim aOption Array
  ReDim aOption(oArgs.Count -1)
  
  'Populate aOption with entries form oArgs
  For i = 0 To oArgs.Count -1
    aOption(i) = Replace(oArgs(i),Chr(10),vbNullString)
  Next
  
  For i = 0 To UBound(aOption)
    s_Log INFORMATION, "[Main Code] aOption(" & i & ")=" & aOption(i)
  Next
  
  s_VerifyOptionAndBuildCommand()
  
Else
  sSrchStr = Replace(oArgs(2),Chr(10),vbNullString)
  'Option not provided setting default line out
  iLn = RESULTDEF
  'sCmd = SEARCHTYPE & " -ln:" & RESULTDEF & " " & Chr(34) & sSrchStr & Chr(34)
  sCmd = SEARCHTYPE & " -ln:" & RESULTDEF & " " & sSrchStr 

End If
    

s_Log INFORMATION, "[Main Code] Maximum number of lines to reply in this search is : " & iLn

'Check presens of important files
f_FileExists LOCATE, True
f_FileExists VFS, True

'Assign current directory\DB as root of DB if not specified in DATABASEROOT
sDBRoot = SCRIPTPATH & "\DB"

'Get the BDs
s_GetDB (sDBRoot)

'Initiate result counter
iRes = 0

iStart = Timer

'Start search in db(s)
For i = 0 To UBound(aDB)
		
	'Stop futher search if result > iLn
	If iRes >= iLn = True Then
		Exit For
	End If
	
	'Make sure Locate doesn't reply with more lines than nessesary (Locate -Ln:num)
	iOut = iLn - iRes
	
	'Searching database
	s_SearchDB aDB(i)
	
Next

iEnd = Timer

'Verify that we have got any result
If f_IsContentInArray = False Then
	s_WriteOutNoResult()
End If

'This loop should be commented out in production, only helpfull during coding/debug
For i = 0 To UBound(aOut)
	s_Log INFORMATION, "[Main Code] aOut(" & i & ")=" & aOut(i)
Next

If CONVERT Or REPLACEVFS Then
	s_AdaptReply()
End If

'This loop should be commented out in production, only helpfull during coding/debug
For i = 0 To UBound(aOut)
	s_Log INFORMATION, "[Main Code] aOut(" & i & ")=" & aOut(i)
Next


'Writes out final output
s_WriteOutResult()

s_DelLog()

Set oArgs = Nothing
Set oFso = Nothing
Set oShell = Nothing

'***************************************************************************************************************************
'                                                     FUNCTION
'***************************************************************************************************************************

Function f_CreateFolder(s)
'************************************************************************
' Purpose : Create a folder, or a subset of folders based upon one path. 
'           Support both path and unc path. Ex : \\fqdn\share\my\path or 
'           D:\share\my\path. If only \\fqdn\share or D:\Share exists, 
'           function loop folder creation until path provided is created.
' Input   : String Path to folder
' Return  : True if folder exists or successfully created
'************************************************************************

Dim sUnc,sResult
	
	On Error Resume Next
	
	Dim sFolder
	
	If Right(s, 1) <> "\" Then
		s = s & "\"
	End If
	
	'Check if path is UNC replace base path \\server.fqdn\something with \
	If Left(s, 2) = "\\" Then
		s_Log INFORMATION, "[Function f_CreateFolder] UNC Path : " & s
		sUNC = s
		Set oRegEx = New RegExp
		oRegEx.Pattern = "\\\\[A-Za-z0-9.]+\\\w+\$?\\"
		sResult = oRegEx.Replace (s, "")
		s_Log INFORMATION, "[Function f_CreateFolder] UNC stripped, result : " & sResult
		sUnc = Left(s, Len(s) - Len(sResult))
		sFolder = sUnc
	End If
	
	s_Log INFORMATION, "[Function f_CreateFolder] Start creating directory structure : " & s
	
	Do Until sFolder = s
		sFolder = Left(s, InStr(Len(sFolder) + 1, s, "\"))
			s_Log INFORMATION, "[Function f_CreateFolder] Checking if folder exists : " & sFolder
			If Not oFso.FolderExists(sFolder) Then
				s_Log INFORMATION, "[Function f_CreateFolder] Creating folder : " & sFolder
				ofso.CreateFolder(sFolder)
				If err.number <> 0 Then
					s_Log ERRORL, "[Function f_CreateFolder] Error creating : " & sFolder & ". The error was : " & err.description
					Exit Function
				End If
			End If
	Loop

	If Right(s, 1) = "\" Then
		s = Left(s, Len(s) -1)
	End If

	f_CreateFolder = True

End Function

Function f_FileExists(s, bRequired)
'*******************************************************
' Purpose : Verify presense of a file, abort the script
'           if the file is not present and it's required
' Input   : s (String) Path to file
'           bRequired (Boolan) True|False
' Return  : True|False Boolean
'*******************************************************

	If (oFso.FileExists(s)) Then
		s_Log SUCCESS, "[f_FileExists] The file : " & s & " is present."
		f_FileExists = True
	Else
		f_FileExists = False
	End If
	
	If bRequired And f_FileExists = False Then
		s_Log ERRORL, "[f_FileExists] The file : " & s & " is NOT present."
		s_Log ERRORL, "[f_FileExists] The file : " & s & " is required for futher processing of this script."
		s_TerminateInSysErr()
	End If
End Function

Function f_GetDayTimeDate()
'****************************************************
' Purpose : Get Day,Date,Time
' Input   : None
' Return  : Current day of week + Date + Time
'****************************************************
	
	f_GetDayTimeDate = WeekDayName(Weekday(Now, vbUseSystemDayOfWeek),,vbUseSystemDayOfWeek) & " " & Date & " "
	
	If CInt(Hour(Now)) < 10 Then
		f_GetDayTimeDate = f_GetDayTimeDate & "0" & Hour(Now)
	Else
		f_GetDayTimeDate = f_GetDayTimeDate & Hour(Now)
	End If
	
	If CInt(Minute(Now)) < 10 Then
		f_GetDayTimeDate = f_GetDayTimeDate & "0" & Minute(Now)
	Else
		f_GetDayTimeDate = f_GetDayTimeDate & Minute(Now)
	End If
	
	If InStr(f_GetDayTimeDate, "/") Then
		f_GetDayTimeDate = Replace(f_GetDayTimeDate, "/", ".")
	End If
	
	If InStr(f_GetDayTimeDate, "\") Then
		f_GetDayTimeDate = Replace(f_GetDayTimeDate, "\", ".")
	End If
	
End Function

Function f_IsContentInArray()
'****************************************************
' Purpose : Verify that oOut is an array with content
' Input   : a (Array)
' Return  : True|False Boolean
'****************************************************

	On Error Resume Next

	If IsArray(aOut) Then
		
		iCnt = UBound(aOut)
		If err.number <> 0 Then
			f_IsContentInArray = False
		Else
			f_IsContentInArray = True
		End If
	Else
		f_IsContentInArray = False
	End If


End Function

Function f_IsInteger(s)
'**************************************************************
' Purpose : Verify if a string is an integer
' Input   : s (String) or expression
' Return  : True|False (Boolean) if conversion was successfull
'**************************************************************

	On Error Resume Next
	
	s = CInt(s)

	If err.number <> 0 Then
			f_IsInteger = False
	Else
			f_IsInteger = True
	End If

End Function

Function f_Randomize(i)
'**************************************************
' Purpose : Create a random integer between 0 and i
' Input   : i (Integer)
' Return  : Random number between 0 and Input (i)
'**************************************************
  
  Dim iInt
  
  iInt = i
  
  s_Log INFORMATION, "[f_Randomize] Creating a random integer between : 0-" & iInt

  Randomize
  iInt = Int(i * Rnd)
  
  s_Log INFORMATION, "[f_Randomize] Random integer is : " & iInt
  
  f_Randomize = iInt
  
End Function

f_SortVFSEntries

Function f_SortVFSEntries(aVFSArray)
'**************************************************
' Purpose : Sort all the vfs entries
' Input   : aVFS (array)
' Return  : the sorted array
'**************************************************
  
  Dim aVFSSorted
  Dim aVfs,sVfs,sPaths,aBuff
  Dim i,startsize
  Dim temp,m,n
  
  s_Log INFORMATION, "[f_SortVFSEntries] Parsing VFS entries."

  'Process array of VFS entries
	For iCnt = 0 To UBound(aVFSArray)
		
		'Get Virtual File System Entry (Ex : /My/FTP)
		aVfs = Split(aVFSArray(iCnt), "|", -1, 1)
		sPaths = aVfs(0)
		sVfs   = aVfs(1)
		
		'If Virtual File System contain multiple path's, split this
		If InStr(sPaths, ";") Then
			aBuff = Split(sPaths,";")
		Else
			aBuff = Array(sPaths)
		End If
		
		'Add to main array
		If IsArray(aVFSSorted) Then
			startsize = UBound(aVFSSorted)+1
			ReDim Preserve aVFSSorted(startsize + UBound(aBuff))
		Else 
		    startsize = 0
		    ReDim aVFSSorted(UBound(aBuff))
		End If
					
		For i = 0 To UBound(aBuff)
			aVFSSorted(startsize+i) = Array(aBuff(i),sVfs)
		Next
	Next

	'Now sort the result
  	s_Log INFORMATION, "[f_SortVFSEntries] Sorting VFS Entries"
	For n = 0 to UBound(aVFSSorted) -1 
		For m = n+1 to UBound(aVFSSorted) 
			If aVFSSorted(m)(0) > aVFSSorted(n)(0) Then
				temp = aVFSSorted(m)
				aVFSSorted(m) = aVFSSorted(n)
				aVFSSorted(n) = temp
			End If
		Next
	Next
	
  f_SortVFSEntries = aVFSSorted
  
End Function
'***************************************************************************************************************************
'                                                        SUB
'***************************************************************************************************************************

Sub s_AdaptReply()
'******************************************************
' Purpose : Adapt/Rewrite reply according to Constant :
'           CONVERT and/or REPLACEVFS
' Input   : None
'******************************************************

	Dim aBuff,aVfs,sVfs,aAllVfs,ia
	
	'Replace path with Virtual File System
	If REPLACEVFS Then
		
		'Get Virtual File System Data from vfs file
		s_GetVFSEntry VFS
		aAllVfs = f_SortVFSEntries(aIndex) 
		
		'Search/Replace oOut array against aBuff array
		For i = 0 To UBound(aOut)
			For ia = 0 To UBound(aAllVfs)
				If (InStr(LCase(aOut(i)), LCase(aAllVfs(ia)(0))) And (aAllVfs(ia)(0) <> vbnullstring)) Then
					aOut(i) = aAllVfs(ia)(1) & Right(aOut(i), Len(aOut(i)) - Len(aAllVfs(ia)(0)))
					Exit For
				End If
			Next
		Next
	End If
	
	'Replace \ with /
	If CONVERT Then
		For iCnt = 0 To UBound(aOut)
			If InStr(aOut(iCnt), "\") Then
				aOut(iCnt) = Replace(aOut(iCnt), "\", "/")
			End If
		Next
	End If

End Sub

Sub s_CreateOutObj()
'*********************************************************************
' Purpose : Create an FTPD reply (Output file) as an object oOut
' Input   : None
'*********************************************************************
	On Error Resume Next

	Set oOut = oFso.CreateTextFile(sOutput & "\ask-onSiteCommand." & sGCID, True)
	
	If err.number <> 0 Then
		s_Log ERRORL, "[s_CreateOutput] Failed to create output : " & sOutput & "\ask-onSiteCommand." & sGCID & ". The error was : " & err.description & " (" & err.number & ")"
		s_TerminateInErr()
	Else
		s_Log INFORMATION, "[s_CreateOutput] File : " & sOutput & "\ask-onSiteCommand." & sGCID & " was successfully created and assigned to object oOut."
	End If
	
End Sub

Sub s_DelLog()
'*********************************************
' Purpose : Delete obsolete logs, this routine
' Input   : None
'*********************************************
	'Delete obsolete search logs
	If LOGS <> 0 Then
		s_Log INFORMATION, "[s_DelLog] Call s_KeepOnlyFiles " & sLogRoot & " " & LOGS
		s_KeepOnlyFiles sLogRoot, LOGS
	End If
	
	'Delete obsolete error logs
	If ERRORLOGS <> 0 Then
		s_Log INFORMATION, "[s_DelLog] Call s_KeepOnlyFiles " & sLogRoot & " " & LOGS
		s_KeepOnlyFiles sErrLogRoot, ERRORLOGS
	End If

	s_Log SUCCESS, "[s_DelLog] " & APPNAME & " " & APPVERMAJOR & "." & APPVERMINOR & " ended successfully"

End Sub

Sub s_DeleteFile(s)
'*************************************
' Purpose : Delete folder
' Input   : String (s) path to folder
'************************************
	On Error Resume Next
	
	err.clear
	
	oFso.DeleteFile s, True
	
	If err.number <> 0 Then
		s_Log ERRORL, "[s_DeleteFile] Failed to delete File : " & s & ". The error was : " & err.description & " (" & err.number & ")"
	Else
		s_Log SUCCESS, "[s_DeleteFile] File : " & s & " was successfully deleted."
	End If

End Sub

Sub s_EventLog(i, s)
'*************************************************************
' Purpose : Write Event to Eventlog.
'           EventLog Level 0 = None 1 = Success/Error 2 = Full
' Input   : i : Event type, Integer
'               0 SUCCESS
'               1 ERROR (Const ERRORL)
'               2 WARNING
'               4 INFORMATION
'           s : Log message, String
'*************************************************************
'Const SUCCESS           = 0
'Const ERRORL            = 1
'Const WARNING           = 2
'Const INFORMATION       = 4
'0 = None 1 = Success/Error 2 = Full


	Select Case EVENTLOG
		Case 1
			Select Case i
				Case SUCCESS
					oShell.LogEvent i, s
				Case ERRORL
					oShell.LogEvent i, s
				Case Else
					Exit Sub
			End Select
		Case 2
			Select Case i
				Case SUCCESS
					oShell.LogEvent i, s
				Case ERRORL
					oShell.LogEvent i, s
				Case WARNING
					oShell.LogEvent i, s
				Case INFORMATION
					oShell.LogEvent i, s
				Case Else
					Exit Sub
			End Select
		Case 3
			Select Case i
				Case SUCCESS
					oShell.LogEvent i, s
				Case ERRORL
					oShell.LogEvent i, s
				Case WARNING
					oShell.LogEvent i, s
				Case INFORMATION
					oShell.LogEvent i, s
				Case Else
					Exit Sub
			End Select
		Case Else
			Exit Sub
	End Select

End Sub

Sub s_FileLog(i, s)
'*********************************************************************************************
' Purpose : Write Event to logfile. Build an array of events until path logfile is determined.
'           If logfile can not be written. Events are redirected to eventlog.
'           EventLog Level 0 = None 1 = Success/Error 2 = Full
' Input   : i : Event type, Integer
'               0 SUCCESS
'               1 ERROR (Const ERRORL)
'               2 WARNING
'               4 INFORMATION
'           s : Log message, String
' Use     : bLog Boolean True|False when actually start logging to file
'*********************************************************************************************
'Const SUCCESS           = 0
'Const ERRORL            = 1
'Const WARNING           = 2
'Const INFORMATION       = 4
'0 = None 1 = Success/Error 2 = Full

On Error Resume Next

Dim iArr
Dim oLog


	'Make sure errorlevel 0
	err.clear
	
	'Log to file if bLog flag is true
	If bLog = True Then
		
		Set oLog = oFso.OpenTextFile(sLog, FORAPPENDING, True)
		
		If err.number <> 0 Then
			iErr = err.number
			'Clear error
			err.clear
			'Check if log folder exists, try to create
			If f_CreateFolder(Left(sLog, InStrRev(sLog, "\"))) Then
				Set oLog = oFso.OpenTextFile(sLog, FORAPPENDING, True)
				
				Else
			
				'Abort filelog operation if can't be written, switch to EventLog	
				EVENTLOG = 2
				FILELOG = 0
				err.Raise = iErr
				s_Log ERRORL, "[Sub s_FileLog] Error occured while writing to logfile : " & sLog & ". The error was : " & err.description
				s_Log WARNING, "[Sub s_FileLog] Events will permanently be redirected to eventlog. All events occured prior to this warning will now be written."
				
				'Write events from array to eventlog
				If IsArray(aLog) AND aLog(0) <> "" Then
					For iArr = 0 To UBound(aLog)
						s_Log CInt(Left(aLog(iArr), 1)), Right(aLog(iArr), Len(aLog(iArr)) - 1)
					Next
					
					ReDim aLog(0)	
				End If
				
				Exit Sub
			End If
		End If
		
		'Write events from array to logfile
		If IsArray(aLog) AND aLog(0) <> "" Then
			For iArr = 0 To UBound(aLog)
				oLog.WriteLine Right(aLog(iArr), Len(aLog(iArr)) - 1)
			Next

			ReDim aLog(0)
		End If
		
		Select Case FILELOG
			Case 1
				Select Case i
					Case SUCCESS
						oLog.WriteLine Now & " SUCCESS    " & s
					Case ERRORL
						oLog.WriteLine Now & " ERROR      " & s
					Case Else
						Exit Sub
				End Select
			Case 2
				Select Case i
					Case SUCCESS
						oLog.WriteLine Now & " SUCCESS     " & s
					Case ERRORL
						oLog.WriteLine Now & " ERROR       " & s
					Case WARNING
						oLog.WriteLine Now & " WARNING     " & s
					Case INFORMATION
						oLog.WriteLine Now & " INFORMATION " & s
					Case Else
						Exit Sub
				End Select
			Case 3
				Select Case i
					Case SUCCESS
						oLog.WriteLine Now & " SUCCESS     " & s
					Case ERRORL
						oLog.WriteLine Now & " ERROR       " & s
					Case WARNING
						oLog.WriteLine Now & " WARNING     " & s
					Case INFORMATION
						oLog.WriteLine Now & " INFORMATION " & s
					Case Else
						Exit Sub
				End Select
			Case Else
				Exit Sub
		End Select
		
		oLog.close
		Set oLog = Nothing
	Else
		
		'Log to array while path to logfile determined
		If IsArray(aLog) Then
			iArr = UBound(aLog) + 1
			ReDim Preserve aLog(iArr)
		Else
			ReDim aLog(1)
		End If
		
		Select Case FILELOG
			Case 1
				Select Case i
					Case SUCCESS
						ReDim Preserve aLog(iArr)
						aLog(iArr) = i & Now & " SUCCESS " & s
					Case ERRORL
						ReDim Preserve aLog(iArr)
						aLog(iArr) = i & Now & " ERROR " & s
					Case Else
						Exit Sub
				End Select
			Case 2
				Select Case i
					Case SUCCESS
						ReDim Preserve aLog(iArr)
						aLog(iArr) = i & Now & " SUCCESS     " & s
					Case ERRORL
						ReDim Preserve aLog(iArr)
						aLog(iArr) = i & Now & " ERROR       " & s
					Case WARNING
						ReDim Preserve aLog(iArr)
						aLog(iArr) = i & Now & " WARNING     " & s
					Case INFORMATION
						ReDim Preserve aLog(iArr)
						aLog(iArr) = i & Now & " INFORMATION " & s
					Case Else
						Exit Sub
				End Select
			Case 3
				Select Case i
					Case SUCCESS
						ReDim Preserve aLog(iArr)
						aLog(iArr) = i & Now & " SUCCESS     " & s
					Case ERRORL
						ReDim Preserve aLog(iArr)
						aLog(iArr) = i & Now & " ERROR       " & s
					Case WARNING
						ReDim Preserve aLog(iArr)
						aLog(iArr) = i & Now & " WARNING     " & s
					Case INFORMATION
						ReDim Preserve aLog(iArr)
						aLog(iArr) = i & Now & " INFORMATION " & s
					Case Else
						Exit Sub
				End Select

			Case Else
				Exit Sub
		End Select
		
	End If

End Sub

Sub s_GetDB(s)
'*********************************************
' Purpose : Get path to all Locate32 databases
'           put this into aDB
' Input   : s (String) Path to root of DB(s)
'*********************************************
	
	On Error Resume Next
	
	Set oFldr = oFso.GetFolder(s)
	
	If err.number <> 0 Then
		s_Log ERRORL, "[s_GetDB] An Error occured connecting to : " & s
		s_TerminateInSysErr()
		Exit Sub
	Else
		s_Log INFORMATION, "[s_GetDB] Connected to : " & s
	End If
	
	Set cFiles = oFldr.Files
	
	If err.number <> 0 Then
		s_Log ERRORL, "[s_GetDB] An Error occured while listing files in folder : " & s
		s_TerminateInSysErr()
		Exit Sub
	Else
		s_Log INFORMATION, "[s_GetDB] Succsessfully listed files in folder : " & s
	End If
	
	iCnt = 0
	
	'Process all files in directory
	For Each oFile in cFiles
		
		'Make sure it's a db before adding path to array
		If LCase(Right(oFile.Name, 4)) = ".dbs" Then
			ReDim Preserve aDB(iCnt)
			aDB(iCnt) = oFile.Path
			sDBTime = cstr(oFile.DateLastModified)
			s_Log INFORMATION, "[s_GetDB] DB added to array : " & aDB(iCnt)
			iCnt = iCnt + 1
		End If
		
	Next

End Sub

Sub s_GetVFSEntry(s)
'************************************************************
' Purpose : Get all entries in FTPD VFS file and
'           populate these into array "aIndex" in lower case
' Input   : s (String) Path to VFS to parse
'************************************************************

	Dim oVfs
	
	s_Log INFORMATION, "[s_GetPathToIndexFromVFS] Reading Virtual File System from " & s & ",  in to array"
	
	'Open Virtual File System
	Set oVfs = oFso.OpenTextFile(s, FORREADING)

	iCnt = 0
	
	'Read Virtual File System, parsing lines
	Do Until oVfs.AtEndOfStream
		sLine = oVfs.ReadLine
		
		'Skip if line contain FTPD specific variables (Ex : Personal home folder)
		If InStr(sLine, "%") Then
			s_Log INFORMATION, "[s_GetPathToIndexFromVFS] " & sLine & " Contain FTPD variables... Line is skipped"
		
		'Processing lines containing path valid to index
		Else
			s_Log INFORMATION, "[s_GetPathToIndexFromVFS] " & sLine & " is valid, parsing..."
		
			ReDim Preserve aIndex(iCnt)
			aIndex(iCnt) = sLine
						
			s_Log INFORMATION, "[s_GetPathToIndexFromVFS] Index entry added to array : " & aIndex(iCnt)
						
			iCnt = iCnt + 1
		End If	
	Loop

End Sub

Sub s_KeepOnlyFiles(s, i)
'*************************************************
' Purpose : Delete obsolete files based upon date.
'           n newest files are kept.
' Input   : String (s) Path to root of files
'           Integer (i) Number of folders to keep
'*************************************************
	On Error Resume Next
	
	Dim aDate
'	Dim oFldr,oFile
'	Dim cFiles
'	Dim iCnt
	Dim n,m,temp
	
	Set oFldr = oFso.GetFolder(s)
	
	If err.number <> 0 Then
		s_Log ERRORL, "[s_KeepOnlyFiles] An Error occured connecting to : " & s & " old files must be deleted manually."
		Exit Sub
	Else
		s_Log INFORMATION, "[s_KeepOnlyFiles] Connected to : " & s
	End If
	
	Set cFiles = oFldr.Files
	
	If err.number <> 0 Then
		s_Log ERRORL, "[s_KeepOnlyFiles] An Error occured while listing files in folder : " & s & ". Old files must be deleted manually."
		Exit Sub
	Else
		s_Log INFORMATION, "[s_KeepOnlyFiles] Succsessfully listed files in folder : " & s
	End If
	
	'Check if number of files greater than i
	If cFiles.Count <= i Then
		s_Log INFORMATION, "[s_KeepOnlyFiles] Number of file(s) in root is : " & cFiles.Count & ". Number is less than or equal to : " & i & " which is the number of file(s) to keep."
		Exit Sub
	End If
		
	s_Log INFORMATION, "[s_KeepOnlyFiles] Number of files in root is : " & cFiles.Count & ". Number is greater than : " & i & " which is the number of file(s) to keep."
	
	'ReDim array to number of folders
	ReDim aDate(cFiles.Count -1)
	
	s_Log INFORMATION, "[s_KeepOnlyFiles] Creating array with fixed size : " & cFiles.Count -1 & ". Adding date of folders to array"
	
	iCnt = 0
	
	'Add date creation of all folders to array
	For Each oFile in cFiles
		s_Log INFORMATION, "[s_KeepOnlyFiles] aDate(" & iCnt & ") = " & oFile.DateCreated & " (Path : " & oFile.Path & ")"
		aDate(iCnt) = oFile.DateCreated
		iCnt = iCnt + 1
	Next
	
	'Sorting array by date, ascent sort order (The famous bubble sort)
	s_Log INFORMATION, "[s_KeepOnlyFiles] Sorting dates in array, ascent sort order"
	For n = 0 to UBound(aDate) -1 
		For m = n+1 to UBound(aDate) 
			If aDate(m) < aDate(n) Then
				temp = aDate(m)
				aDate(m) = aDate(n)
				aDate(n) = temp
			End If
		Next
	Next
	
	'Just logging
	s_Log INFORMATION, "[s_KeepOnlyFiles] Array sorted, result in ascent order : "
	For iCnt = 0 To UBound(aDate)
		s_Log INFORMATION, "[s_KeepOnlyFiles] aDate(" & iCnt & ") = " & aDate(iCnt)
	Next
			
	
	
	'Cut array, keep (i) rows with date of files which are going to be deleted
	ReDim Preserve aDate(UBound(aDate) -i)
	s_Log INFORMATION, "[s_KeepOnlyFiles] Array aDate cut, remaing rows are : "
	
	'Even more logging (those debuggers ;))
	For iCnt = 0 To UBound(aDate)
		s_Log INFORMATION, "[s_KeepOnlyFiles] aDate(" & iCnt & ") = " & aDate(iCnt)
	Next
	
	s_Log INFORMATION, "[s_KeepOnlyFiles] Start deleting file(s)..."
	
	For Each oFile in cFiles
		For iCnt = 0 To UBound(aDate)
			If oFile.DateCreated = aDate(iCnt) Then
				s_Log INFORMATION, "[s_KeepOnlyFiles] aDate(" & iCnt & ") = " & aDate(iCnt) & " matches DateCreated on file : " & oFile.Path & " (Date=" & oFile.DateCreated & ")"
				s_DeleteFile oFile.Path
				Exit For
			End If
		Next
	Next
End Sub

Sub s_Log(i, s)
'***********************************************************************************************
' Purpose : Log interface, write various type of events to different formats, NT Event, File etc
'           Easily extend logging into other formats, Ex : Syslog, mail, xml etc.
' Input   : i : Event type, Integer 0 = None 1 = Success/Error 2 = Full
'           s : Log message, String
'***********************************************************************************************
	
	'Eventlog output
	If EVENTLOG = 1 or EVENTLOG = 2 Then
		Set oRegEx = New RegExp
		oRegEx.Pattern = "\[.*\] "
		s = oRegEx.Replace (s, "")
		s_EventLog i, s
		Set oRegEx = Nothing
	End If
	
	'Debug EventLog output
	If EVENTLOG = 3 Then
		s_EventLog i, s
	End If

	'Logfile output
	If FILELOG = 1 or FILELOG = 2 Then
		Set oRegEx = New RegExp
		oRegEx.Pattern = "\[.*\] "
		s = oRegEx.Replace (s, "")
		s_FileLog i, s
		Set oRegEx = Nothing
	End If
	
	'Debug logfile output
	If FILELOG = 3 Then
		s_FileLog i, s
	End If	
	
	'Debug screen output
	If FILELOG = 3 or EVENTLOG = 3 Then
		wscript.echo s
	End If
	
End Sub

Sub s_MoveFile(s, sDest)
'*********************************************************
' Purpose : Move a file from one directory to another
' Input   : s (String) Full path to file. Ex : C:\Move.me
'           sDest (String) Full path to destination folder
'           Ex : C:\Move
'*********************************************************
	
	On Error Resume Next
	
	If Right(sDest, 1) <> "\" Then
		sDest = sDest & "\"
	End If
	
	oFso.MoveFile s, sDest

End Sub

Sub s_SearchDB(s)
'******************************************************
' Purpose : Search a Locate32 DB, put result into array
' Input   : s (String) Path to DB
'******************************************************

	Dim a,oExec,oStdOut
	
	s_Log INFORMATION, "[s_SearchDB] Searching Locate32 DB : " & s & ", with string : " & sSrchStr
	s_Log INFORMATION, "[s_SearchDB] Syntax : " & LOCATE & " -R -d " & Chr(34) & s & Chr(34) & " " & sCmd

	Set oExec = oShell.Exec(LOCATE & " -R -d " & Chr(34) & s & Chr(34) & " " & sCmd)

	'Create sdtout object for redirect of output from locate.exe
	Set oStdOut = oExec.StdOut

	'Grabbing output from Locate32
	s_Log INFORMATION, "[s_SearchDB] Redirecting output from : " & LOCATE
	
	Do Until oStdOut.AtEndOfStream

		If iRes >= iLn = True Then
			Exit Do
		End If
				
		sLine = oStdOut.ReadLine
		s_Log INFORMATION, "[s_SearchDB] Locate.exe says : " & sLine
		
		'Populating array aOut with result
		ReDim Preserve aOut(iRes)
		'aOut(iRes) = LCase(sLine)
		aOut(iRes) = sLine
		
		iRes = iRes + 1
	Loop
	
	Set oExec = Nothing
	Set oStdOut = Nothing
End Sub

Sub s_TerminateInErr()
'****************************************
' Puropose : Terminate the script, moving
'            log to error directory
' Input    : None
'****************************************
	s_Log ERRORL, "[s_TerminateErr] " & ABORT
	s_MoveFile sLog, sErrLogRoot
	wscript.quit

End Sub

Sub s_TerminateInHlp()
'***********************************************************
' Puropose : Terminate the script, write help output to user
' Input    : None
'***********************************************************

  s_WriteOutPre()

'All options from Locate.exe, added because of laziness =) during development...

'parameters:
'    -s          pause after page
'    -i          shows what are we looking for
'    -p          look in directory 'path'
'    -r          'file' is interpreted as regular expression
'    -w          use whole path, not just file name
'    -t          set extension
'        -R                      search subdirectories
'        -Rn                     don't search subdirectories
'    -d file     set database file
'    -D name     uses database named 'name'
'    -lm:SIZE    set minimum size to SIZE, eg -lms:100k
'    -lM:SIZE    set maximum size to SIZE, eg -lMs:10M
'    -lDmDAY     set minimum file modification date, day can be exact date in the
'
'                form of YYMMDD, or number (0 is today, 1 is yesterday, etc)
'    -lDcDAY     set minimum file creation date
'    -lDaDAY     set minimum file last access date
'    -lDMDAY     set maximum file modification date
'    -lDCDAY     set maximum file creation date
'    -lDADAY     set maximum file last access date
'    -ln:NUM     set number of maximum found files
'    -lf         look only file names (default)
'    -ld         look only directory names
'    -lfd        look file and directory names
'    -lc:TEXT    file must contain string TEXT
'    -lcm        string text is match case (default)
'    -lcnm       string text is not match case
'    -lR:TEXT    same as above but TEXT is regular expression
'    -lw         match whole name only
'    -lrn        don't replace spaces with asterisk
'    -v                  verbose mode, information about search
'    -h          help, this screen
	
	If SCENEVIEW Then
	
  	With oOut
    	.WriteLine "| Syntax :                                                                     |"
    	.WriteLine "| Search Filename|Dirname [Options] or help|-h or showme                       |"
    	.WriteLine "|                                                                              |"
    	.WriteLine "| Filename      Required, name of file (string)                                |"
    	.WriteLine "| Dirname       Required, name of directory (string)                           |"
    	.WriteLine "| -t            Optional, set file extension (Used with -lf)                   |"
    	.WriteLine "| -lm:SIZE      Optional, set minimum size to SIZE, ex -lm:100K                |"
        .WriteLine "| -lM:SIZE      Optional, set maximum size to SIZE, ex -lm:100M                |"
    	.WriteLine "| -ln:NUM       Optional, set number of maximum lines in result                |"
    	.WriteLine "| -lf           Optional, search only file names                               |"
    	.WriteLine "| -ld           Optional, search only directory names                          |"
    	.WriteLine "| -lfd          Optional, search file and directory names                      |"
    	.WriteLine "| -lc:TEXT      Optional, file must contain text (used with -lf)               |"
    	.WriteLine "| -lcmn         Optional, string text do NOT match case (default do)           |"
    	.WriteLine "| -lrn          Optional, don't replace spaces with asterisk (default do)      |"
    	.WriteLine "|                                                                              |"
    	.WriteLine "| help          Show this help                                                 |"
    	.WriteLine "| showme        Show examples                                                  |"
    	.WriteLine "|                                                                              |"
    	.WriteLine "|               !!!OPTIONS ARE CASE SENSITIVE!!!                               |"
    	.WriteLine "|                                                                              |"
    	.WriteLine "| Example :                                                                    |"
    	.WriteLine "| site search myfile.txt -lf -ln:50                                            |"
    	.WriteLine "| site search myfile -tzip -lf                                                 |"
    	.WriteLine "| site search " & Chr(34) & Chr(34) & " -lc:text -lcmn                                                |"
    	.WriteLine "| site search " & Chr(34) & "something with space" & Chr(34) & "                                           |"
    	.WriteLine "|                                                                              |"
    End With
  
  Else
  
  	With oOut
  	.WriteLine "Syntax :"
  	.WriteLine "Search Filename|Dirname [Options] or help|-h or showme"
  	.WriteBlankLines 1
  	.WriteLine " Filename      Required, name of file (string)"
  	.WriteLine " Dirname       Required, name of directory (string)"
  	.WriteLine " -t            Optional, set file extension (Used with -lf)"
  	.WriteLine " -lm:SIZE      Optional, set minimum size to SIZE, ex -lm:100K"
    .WriteLine " -lM:SIZE      Optional, set maximum size to SIZE, ex -lm:100M"
  	.WriteLine " -ln:NUM       Optional, set number of maximum lines in result"
  	.WriteLine " -lf           Optional, search only file names"
  	.WriteLine " -ld           Optional, search only directory names"
  	.WriteLine " -lfd          Optional, search file and directory names"
  	.WriteLine " -lc:TEXT      Optional, file must contain text (used with -lf)"
  	.WriteLine " -lcmn         Optional, string text do NOT match case (default do)"
  	.WriteLine " -lrn          Optional, don't replace spaces with asterisk (default do)"
  	.WriteLine ""
  	.WriteLine " help          Show this help"
  	.WriteLine " showme        Show examples"
  	.WriteBlankLines 1
  	.WriteLine " NOTE! OPTIONS ARE CASE SENSITIVE"
    .WriteBlankLines 1
  	.WriteLine " Example :"
  	.WriteLine " site search myfile.txt -lf -ln:50"
  	.WriteLine " site search myfile -tzip -lf"
  	.WriteLine " site search " & Chr(34) & Chr(34) & " -lc:text -lcmn"
  	.WriteLine " site search " & Chr(34) & "something with space" & Chr(34)
    .WriteBlankLines 1
  End With
  
  End If
  
  s_WriteOutPost()
  
  s_DelLog()
	
	wscript.quit
	
End Sub

Sub s_TerminateInShowMe()
'******************************************************************
'Purpose : Show FTPD User command examples for various search types
'******************************************************************

  s_WriteOutPre()
  
  If SCENEVIEW Then
    oOut.WriteLine "| Site search command examples based on -ld as default option"
    oOut.WriteLine "| "
    oOut.WriteLine "| "    
    oOut.WriteLine "| Site search command examples"
    oOut.WriteLine "| "
    oOut.WriteLine "| site search dr*awesome -ln:5 -lM:100K"
    oOut.WriteLine "| Search for folders (-ld is default) containing 'dr' and 'awesome' (*=wildcard)"
    oOut.WriteLine "| Max search results 5 (ln:5) and max size of file 100kb (-lM:100K)"
    oOut.WriteLine "| "
    oOut.WriteLine "| site search dr*awesome -ln:5 -lm:100K"
    oOut.WriteLine "| Search for folders (-ld is default) containing 'dr' and 'awesome' (*=wildcard)"
    oOut.WriteLine "| Max search results 5 (ln:5) and min size of file 100kb (-lm:100K)"
    oOut.WriteLine "| "
    oOut.WriteLine "| site search dr*awesome -ln:5 -lM:100K -tmod"
    oOut.WriteLine "| Search for folders (-ld is default) containing 'dr' and 'awesome' (*=wildcard)"
    oOut.WriteLine "| Max search results 5 (ln:5) and max size of file 100kb (-lM:100K). Only files with .mod extension (-tmod)"
    oOut.WriteLine "| "
    oOut.WriteLine "| site search dr*awesome -lf -ln:5 -lM:100K"
    oOut.WriteLine "| Search for files (-lf) containing 'dr' and 'awesome' (*=wildcard)"
    oOut.WriteLine "| Max search results 5 (ln:5) and max size of file 100kb (-lM:100K)"
    oOut.WriteLine "| "
    oOut.WriteLine "| site search dr*awesome -ldf -ln:5 -lM:100K -tmod"
    oOut.WriteLine "| Search for files and folders (-ldf) containing the text 'dr' and 'awesome' (*=wildcard)"
    oOut.WriteLine "| Max search results 5 (ln:5) and max size of file 100kb (-lM:100K). Only files with .mod extension (-tmod)"
    'oOut.WriteLine "| "
    'oOut.WriteLine "| Check docs at this address http://sourceforge.net/docman/?group_id=181483 for more search tips."
    
  Else
    
    oOut.WriteLine " Site search command examples based on -ld as default option"
    oOut.WriteBlankLines 2
    oOut.WriteLine " Site search command examples"
    oOut.WriteBlankLines 1
    oOut.WriteLine " site search dr*awesome -ln:5 -lM:100K"
    oOut.WriteLine " Search for folders (-ld is default) containing 'dr' and 'awesome' (*=wildcard)"
    oOut.WriteLine " Max search results 5 (ln:5) and max size of file 100kb (-lM:100K)"
    oOut.WriteBlankLines 1
    oOut.WriteLine " site search dr*awesome -ln:5 -lm:100K"
    oOut.WriteLine " Search for folders (-ld is default) containing 'dr' and 'awesome' (*=wildcard)"
    oOut.WriteLine " Max search results 5 (ln:5) and min size of file 100kb (-lm:100K)"
    oOut.WriteBlankLines 1
    oOut.WriteLine " site search dr*awesome -ln:5 -lM:100K -tmod"
    oOut.WriteLine " Search for folders (-ld is default) containing 'dr' and 'awesome' (*=wildcard)"
    oOut.WriteLine " Max search results 5 (ln:5) and max size of file 100kb (-lM:100K). Only files with .mod extension (-tmod)"
    oOut.WriteBlankLines 1
    oOut.WriteLine " site search dr*awesome -lf -ln:5 -lM:100K"
    oOut.WriteLine " Search for files (-lf) containing 'dr' and 'awesome' (*=wildcard)"
    oOut.WriteLine " Max search results 5 (ln:5) and max size of file 100kb (-lM:100K)"
    oOut.WriteBlankLines 1
    oOut.WriteLine " site search dr*awesome -ldf -ln:5 -lM:100K -tmod"
    oOut.WriteLine " Search for files and folders (-ldf) containing the text 'dr' and 'awesome' (*=wildcard)"
    oOut.WriteLine " Max search results 5 (ln:5) and max size of file 100kb (-lM:100K). Only files with .mod extension (-tmod)"
    'oOut.WriteBlankLines 1
    'oOut.WriteLine " Check docs at this address http://sourceforge.net/docman/?group_id=181483 for more search tips"
    
  End If
  
  s_WriteOutPost()

End Sub

Sub s_TerminateInSysErr()
'****************************************************************
' Puropose : Terminate the script, moving log to error directory.
'            Inform the user about the error.
' Input    : None
'****************************************************************
	s_Log ERRORL, "[s_TerminateInSysErr] " & ABORT
	
	s_WriteOutPre()
	
	If SCENEVIEW Then
	
    oOut.WriteLine "| An internal system error has occured. The error has been logged, please      |"
    oOut.WriteLine "| report this error to system administrator.                                   |"
	
  Else
  
    oOut.WriteLine "An internal system error has occured. The error has been logged, please report"
    oOut.WriteLine "this error to system administrator."
  
  End If
	
	s_WriteOutPost()
	
	s_Log ERRORL, "[f_FileExists] " & ABORT
	
	s_MoveFile sLog, sErrLogRoot
	wscript.quit

End Sub

Sub s_VerifyOptionAndBuildCommand()
'****************************************************************
'Purpose : Verify options provided by user. Options are provided
'          in array aOption. Every option verified, is checked
'          out of array. If all options are verified successfully
'          a command string to pass to Locate32 is build. If one
'          or more options fail to verify, user is redirected to
'          help.
'****************************************************************
  
  Dim bSrchType,bLn,bMatchWholeName,bSetExtension,bContainTxt,bAsterisk,bStrNotMatchCase
  Dim bMinSize,bMaxSize
  
  'Flag rows allready checked in main code as True
  aOption(0) = True
  aOption(1) = True
  
  'Get search string
  sSrchStr = aOption(2)
  
  'Check out search string
  aOption(2) = True
  
  'Loop through rest of array
  For iCnt = 0 To UBound(aOption)
    
    If aOption(iCnt) <> True Then
      
      'Check if option is "Search Type"
      If bSrchType <> True Then
      
        Select Case aOption(iCnt)
          
          Case "-lfd"
            sCmd = sCmd & " -lfd"
            aOption(iCnt) = True
            bSrchType = True
            s_Log INFORMATION, "[s_VerifyOptionAndBuildCommand] User requested : files & directory, as search type."

          
          Case "-ldf"
            sCmd = sCmd & " -lfd"
            aOption(iCnt) = True
            bSrchType = True
            s_Log INFORMATION, "[s_VerifyOptionAndBuildCommand] User requested : files & directory, as search type."


          Case "-ld"
            sCmd = sCmd & " -ld"
            aOption(iCnt) = True
            bSrchType = True
            s_Log INFORMATION, "[s_VerifyOptionAndBuildCommand] User requested : directory, as search type."


          Case "-lf"
            sCmd = sCmd & " -lf"
            aOption(iCnt) = True
            bSrchType = True
            s_Log INFORMATION, "[s_VerifyOptionAndBuildCommand] User requested : files, as search type."

        End Select
      End If
      
      'Check if option is "Number of Lines"
      If bLn <> True Then
        
        If InStr(aOption(iCnt), "-ln:") Then
        
          'Filtering out -ln: leaving ln with integer only
		      iLn = Right(aOption(iCnt), Len(aOption(iCnt)) -4)
		    
		    		'Prevent user from sending a large junk of numbers causing overflow 
        		If Len(iLn) > Len(RESULTMAX) Then
        			iLn = RESULTMAX
        			s_Log WARNING, "[s_VerifyOptionAndBuildCommand] The number of lines option provided by user is way to high. Set number of lines to system Maximum to prevent possible overflow."
        			sCmd = sCmd & " -ln:" & RESULTMAX
              aOption(iCnt) = True
              bLn = True
            
            'Prevent system to output more lines than permitted if asked by user
            ElseIf CLng(iLn) > RESULTMAX Then
        			iLn = RESULTMAX
        			s_Log WARNING, "[s_VerifyOptionAndBuildCommand] The number of lines option provided by user is to high. Set number of lines to system Maximum."
        			sCmd = sCmd & " -ln:" & RESULTMAX
              aOption(iCnt) = True
              bLn = True            
            
            'Convert iLn safely to integer
            Else          		
              iLn = CLng(iLn)
              sCmd = sCmd & " -ln:" & iLn
              aOption(iCnt) = True
              bLn = True
              s_Log INFORMATION, "[s_VerifyOptionAndBuildCommand] User requested : " & iLn & " number of lines in reply as option"
        		End If
		    
		    End If
    
      End If      
      
      'Check if option is "Match Whole Name Only"
      If bMatchWholeName <> True And aOption(iCnt) = "-lw" Then
        sCmd = sCmd & " -lw"
        bMatchWholeName = True
        aOption(iCnt) = True
        s_Log INFORMATION, "[s_VerifyOptionAndBuildCommand] User requested search to : Match Whole Name Only"
      End If
      
      'Check if option is "Set extension"
      If bSetExtension <> True And Left(aOption(iCnt),2) = "-t" Then
        sCmd = sCmd & " " & aOption(iCnt)
        bSetExtension = True
        s_Log INFORMATION, "[s_VerifyOptionAndBuildCommand] User requested to set search extension to : " & Right(aOption(iCnt), Len(aOption(iCnt)) -2)
        aOption(iCnt) = True
      End If

      'Check if option is "must contain text"
      If bContainTxt <> True And Left(aOption(iCnt),4) = "-lc:" Then
        sCmd = sCmd & " " & aOption(iCnt)
        bContainTxt = True
        s_Log INFORMATION, "[s_VerifyOptionAndBuildCommand] User requested that result must contain text : " & Right(aOption(iCnt), Len(aOption(iCnt)) -4)
        aOption(iCnt) = True
      End If

      'Check if option is "Replace space with asterisk"
      If bAsterisk <> True And aOption(iCnt) = "-lrn" Then
        sCmd = sCmd & " " & aOption(iCnt)
        bAsterisk = True
        s_Log INFORMATION, "[s_VerifyOptionAndBuildCommand] User requested that spaces in string should NOT be replaced by asterisk"
        aOption(iCnt) = True
      End If

      'Check if option is "Do NOT match case in text"
      If bStrNotMatchCase <> True And aOption(iCnt) = "-lcnm" Then
        sCmd = sCmd & " " & aOption(iCnt)
        bStrNotMatchCase = True
        s_Log INFORMATION, "[s_VerifyOptionAndBuildCommand] User requested that text string is NOT case sensitive"
        aOption(iCnt) = True
      End If

      'Check if option is "Minimum size"
      If bMinSize <> True And Left(aOption(iCnt),4) = "-lm:" Then
        sCmd = sCmd & " " & aOption(iCnt)
        bMinSize = True
        s_Log INFORMATION, "[s_VerifyOptionAndBuildCommand] User requested that minimum filesize is : " & Right(aOption(iCnt), Len(aOption(iCnt)) -4)
        aOption(iCnt) = True
      End If

      'Check if option is "Maximum size"
      If bMaxSize <> True And Left(aOption(iCnt),4) = "-lM:" Then
        sCmd = sCmd & " " & aOption(iCnt)
        bMaxSize = True
        s_Log INFORMATION, "[s_VerifyOptionAndBuildCommand] User requested that maximum filesize is : " & Right(aOption(iCnt), Len(aOption(iCnt)) -4)
        aOption(iCnt) = True
      End If

    End If
    
    'Not an option : must be continued search string
    If (aOption(iCnt) <> True) Then
    	sSrchStr = sSrchStr & " " & aOption(iCnt)
    	aOption(iCnt) = True
    End If   
  
  Next
  
  'Verify all options are true
  For iCnt = 0 To UBound(aOption)
    'If not true, redirect user to help
    If aOption(iCnt) <> True Then
      s_Log WARNING, "[s_VerifyOptionAndBuildCommand] Row : " & iCnt & ", in aOption contained : " & aOption(iCnt) & " NOT a valid option, redirect user to help"
      s_TerminateInHlp()
    End If
  Next
  
  'Assign default search type if not provide by user in option
  If bSrchType <> True Then
    s_Log SUCCESS, "[s_VerifyOptionAndBuildCommand] User did NOT provide any search type option, set to system default"
    sCmd = sCmd & " " & SEARCHTYPE
  End If
  
  'Assign default number of lines in output if not provided by user in option
  If bLn <> True Then
    s_Log SUCCESS, "[s_VerifyOptionAndBuildCommand] User did NOT provide any number of lines option, set to system default"
    iLn = RESULTDEF
    sCmd = sCmd & " -ln:" & RESULTDEF
  End If
  
  'Assign search string to command
  'sCmd = sCmd & " " & Chr(34) & sSrchStr & Chr(34)
  sCmd = sCmd & " " & sSrchStr 
  
  'Trim possible leading and trailing spaces
  sCmd = Trim(sCmd)
  
  s_Log SUCCESS, "[s_VerifyOptionAndBuildCommand] Command build successfully, result : " & sCmd
  
End Sub

Sub s_WriteOutPre()
'******************************************************
' Puropose : Write the common intro of every user reply
' Input    : None
'******************************************************
  
  Dim iVerStr,sVerStr,sStat
  
  oOut.WriteLine "1"
  
  If SCENEVIEW Then
  	 sVerStr = SCENEHEADER
  	 oOut.WriteLine SCENEHEADER
  Else
     sVerStr = NORMALHEADER
     oOut.WriteLine sVerStr
     'oOut.WriteBlankLines 1
  End If
End Sub

Sub s_WriteOutNoResult()
'*************************************************************
' Puropose : Reply to user that search ended with empty result
' Input    : None
'*************************************************************
  
  Dim sStat,sStatstart,sStatend,sDBTimeMsg
  
	s_WriteOutPre()
	
	If SCENEVIEW Then
	
		If SEARCHDETAILS then
			'Build statistics/fact string
		    sStat = "| Search: " & Chr(34) & sSrchStr & Chr(34) 
		    
		    sDBTimeMsg = "DB[" & sDBTime & "] "
		    
		    If ((Len(sStat + sDBTimeMsg) +  1) > WIDTH) Then
		    	sStat = "| Search: " & Chr(34) & Left(sSrchStr,(Len(sSrchStr) - ((Len(sStat + sDBTimeMsg) +2) - WIDTH))) & Chr(34)
		    End If 
		    
		    For iCnt = 1 To (WIDTH - Len(sStat + sDBTimeMsg) - 1)
		  	  sStat = sStat & " "
		  	Next
		    sStat = sStat & sDBTimeMsg & "|" 
		  	
		    oOut.WriteLine sStat
		    oOut.WriteLine SCENESEPARATOR 
		    
		    sStatend = "[" & FormatNumber(iEnd -iStart, 4) & " sec] "
	    End If 
	    sStatstart = "| No Result."     
	        
	    For iCnt = 1 To WIDTH -Len(sStatstart + sStatend) -1
	  	  sStatstart = sStatstart & " "
	  	Next
	  	sStat = sStatstart & sStatend & "|"
		oOut.WriteLine sStat
	    		
	Else
	
		If SEARCHDETAILS then
			'Build statistics/fact string
		    sStat = " Search: " & Chr(34) & sSrchStr & Chr(34) & ", Time used : " & FormatNumber(iEnd -iStart, 4) & " sec."
		    oOut.WriteLine sStat
		    oOut.WriteLine NORMALSEPARATOR
	  	End If
		oOut.WriteLine " No Result."
		'oOut.WriteBlankLines 1  
		
  End If
		
	s_WriteOutPost()
	s_DelLog()
	wscript.quit
	
End Sub

Sub s_WriteOutPost()
'***********************************************************
' Puropose : Write the common last lines of every user reply
' Input    : None
'***********************************************************
	Dim sDefSrch,sDef,sMax,sFullstr, sHelpstr 
	
	If SCENEVIEW Then
		If SEARCHDETAILS then
		  	'Build default search string
		  	If LCase(SEARCHTYPE) = "-lf" Then
		  	  sDefSrch = "| Def.Search: File "
		  	ElseIf LCase(SEARCHTYPE) = "-ld" Then
		  	  sDefSrch = "| Def.Search: Dir "
		  	Else
		  	  sDefSrch = "| Def.Search: Dir & Files "
		  	End If
		  	
		  	'Build default result string
		  	sDef = "/ " & RESULTDEF & " "
		  	'Build max result string
		  	sMax = "/ " & RESULTMAX
		  	sHelpstr = "help : <site search help> "
		  	
		  	'Build Full String
		  	sFullstr = sDefSrch + sDef + sMax
		  	sFullstr = sFullstr & Space(WIDTH - Len(sFullstr & sHelpstr) -1) & sHelpstr & "|"
		  	  	  	
		    oOut.WriteLine SCENESEPARATOR
		    oOut.WriteLine sFullstr
		 End If 
    oOut.WriteLine SCENETAIL
  	oOut.Close

  Else
	If SEARCHDETAILS then
	  	'Build default search string
	  	If LCase(SEARCHTYPE) = "-lf" Then
	  	  sDefSrch = " Def.Search: File."
	  	ElseIf LCase(SEARCHTYPE) = "-ld" Then
	  	  sDefSrch = " Def.Search: Dir."
	  	Else
	  	  sDefSrch = " Def.Search: Dir & Files."
	  	End If
	    
	    'Build default result string
	    sDef = " Def.Results: " & RESULTDEF & " items."
	    
	    'Build max result string
		sMax = " Max.Results: " & RESULTMAX & " items."
	    
	    oOut.WriteLine NORMALSEPARATOR
	    oOut.WriteLine sDefSrch
	    oOut.WriteLine sDef
	    oOut.WriteLine sMax
	    'oOut.WriteBlankLines 1
    End If 
    oOut.WriteLine  NORMALTAIL
	oOut.Close
    
  End If
	
End Sub

Sub s_WriteOutResult()
'********************************
' Puropose : Reply result to user
' Input    : None
'********************************
	
	Dim sStat,sResult,sStatstart,sStatend,sDBTimeMsg
	
	s_WriteOutPre()
	
	If SCENEVIEW Then
	
		If SEARCHDETAILS Then	
			'Build statistics/fact string
		    sStat = "| Search: " & Chr(34) & sSrchStr & Chr(34) 
		    
		    sDBTimeMsg = "DB[" & sDBTime & "] "
		    
		    If ((Len(sStat + sDBTimeMsg) +  1) > WIDTH) Then
		    	sStat = "| Search: " & Chr(34) & Left(sSrchStr,(Len(sSrchStr) - ((Len(sStat + sDBTimeMsg) +2) - WIDTH))) & Chr(34)
		    End If 
		
		    sStat = sStat & Space(WIDTH - Len(sStat + sDBTimeMsg) - 1) & sDBTimeMsg & "|" 
		  	
		    oOut.WriteLine sStat
		    oOut.WriteLine SCENESEPARATOR
			
			sStatstart = "| Found " & UBound(aOut) + 1 & " Result(s):"     
		    sStatend = "[" & FormatNumber(iEnd -iStart, 4) & " sec] "
		  	
		  	sStat = sStatstart & Space(WIDTH - Len(sStatstart + sStatend) -1) & sStatend & "|"
			
			oOut.WriteLine sStat
			'oOut.WriteLine "|" & Space(WIDTH - 2) & "|"
		End If 
		
		For i = 0 To UBound(aOut)
	  		'Create ending frame if result is less than 80 characters
	  		If Len("| " & aOut(i)) < WIDTH Then
	  		  sResult = "| " & aOut(i)
	  		  For iCnt = 1 To WIDTH - Len("| " & aOut(i)) -1
	  		    sResult = sResult & " "
	  		  Next
	  		  sResult = sResult & "|"
	  		  oOut.WriteLine sResult
	  		 Else
	  			'Skip ending frame if result is greater than WIDTH characters
	  			oOut.WriteLine "| " & aOut(i)
	  		End If
	  	Next
	  	'oOut.WriteLine "|" & Space(WIDTH - 2) & "|"
   Else
  	
  	For i = 0 To UBound(aOut)
  		oOut.WriteLine " " & aOut(i)
  	Next
    
    If SEARCHDETAILS then
	    'Build statistics/fact string
	    sStat = " Search: " & Chr(34) & sSrchStr & Chr(34) & ", Returned: " & UBound(aOut) + 1 & ", Time used : " & FormatNumber(iEnd -iStart, 4) & " sec."
	       
	    oOut.WriteBlankLines 1
	    oOut.WriteLine sStat
    End If 
  End If

  s_WriteOutPost()
  s_DelLog()
  WScript.quit
End Sub

Sub Import (sFileName)
	'**********************************************************************************
	' Purpose : Import external file
	'**********************************************************************************
	Dim oFso, oFile
	Dim sScriptDir, sFilePath, sStreamData
	
	Set oFso		= CreateObject("Scripting.FileSystemObject")
	sScriptDir = oFso.GetParentFolderName(WScript.ScriptFullName)   
	sFilePath = oFso.BuildPath(sScriptDir,sFileName)   
	Const ForReading = 1   
	
	Set oFile = oFso.OpenTextFile(sFilePath,ForReading)   
	sStreamData = oFile.ReadAll()   
	ExecuteGlobal sStreamData   
End Sub